/*****************************************************************************
 *    rand.c : write a randomization of files or stdin or parms to stdout
 *    Usage:
 *        blah | rand
 *        rand -f <file>
 *        rand <parms to be randomized>
 * 	
 *     Copyright (C) 1998 Erik Greenwald <br0ke@math.smsu.edu>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ******************************************************************************/

#define VERSION "1.2"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

   /*** struct for the linked lists ***/
struct ll{
	char *data;
	struct ll *next;
};

   /*** scramble function, for pipe and files (not parm) ***/

void scramble(FILE *fp){
	char *blah, **table;
	struct ll *llist, *ptr;
	int size=0, n, x;

	llist= malloc(sizeof(struct ll));
	llist->data=NULL;
	ptr=llist;

	blah=(char *)malloc(sizeof(char)*1024);
	
   /*** make linked list     ***/
	while(fgets(blah,1024,fp))
	{
		struct ll *x=malloc(sizeof(struct ll));
		x->data=malloc(strlen(blah)*sizeof(char)+2);
		memcpy(x->data,blah,strlen(blah)*sizeof(char)+1);
		x->next=NULL;
		ptr->next=x;
		ptr=x;
		size++;
	}
 
   /*** make table from list ***/

	table=malloc(size*sizeof(void *));
	ptr=llist->next;
	for(x=0;x<size;x++)
	{
		table[x]=ptr->data;
		ptr=ptr->next;
	}

  /* shuffle it    (Dr Shades algorithm) */
  n = size;

  while (n > 1)
  {
    int d = ((double)rand()/RAND_MAX) * n;
    char *temp = table[d];
    table[d] = table[n-1];
    table[n-1] = temp;
    --n;
  }
  
   /*** print it   ***/

	while(size)
	{
		fprintf(stdout,"%s",table[size - 1]);
		size--;
	}

   /*** delete the linked list and clean up ***/
	
	ptr=llist->next;
	while(ptr)
	{
		free(llist);
		llist=ptr;
		ptr=ptr->next;
	}
	return;
}

/***********************************************************************/

void show_help(char *bleh)
{
	fprintf(stdout,"%s %s(originally rand %s) (C) 1998 Erik Greenwald\n%s comes with ABSOLUTELY NO WARRANTY. Please read the GPL for details.\n\nUsage:\n\t<command> | %s\n\t%s -f <file>\n\t%s <things to mix up>\n\n",bleh,VERSION, VERSION,bleh,bleh,bleh,bleh);
}

void show_version(char *bleh)
{
	fprintf(stdout,"%s %s (originally rand %s) (C) 1998 Erik Greenwald\n\n",bleh,VERSION,VERSION);
}
/***********************************************************************/

int main(int argc, char **argv)
{
	int x, c;
	double d;
	srand(getpid());

		/* |rand */
	if(argc<=1)scramble(stdin);

		/* rand -f <file> */
	else if(!strncmp(argv[1],"-f",2))
	{
		FILE *fp;
		if((fp=fopen(argv[2],"r"))!=NULL)scramble(fp);
		else {
			fprintf(stderr,"File not found\n");
			exit(-1);
		}
		fclose(fp);
	}

		/* rand a b c d e f g */
	else if(argv[1][0]!='-')
	{
		for(x=0;x<argc;x++)argv[x]=argv[x+1];
		argc--;
		while(argc>1)
		{
			d=(double)rand()/RAND_MAX;
			c=d*argc-1;
			fprintf(stdout,"%s\n",argv[c]);
			for(x=c;x<argc;x++)argv[x]=argv[x+1];
			argc--;
		}
	}

	else if(!strncmp(argv[1],"-h",2)||!strncmp(argv[1],"--help",6))
		show_help(argv[0]);
	else if(!strncmp(argv[1],"-v",2)||!strncmp(argv[1],"--version",4))
		show_version(argv[0]);
	else show_help(argv[0]);	/* unknown - command */
	
	return 0;
}
